module Gleis
  module CLI
    # Application-related CLI commands
    class App < Thor
      class_option :app, aliases: :'-a', type: :string, default: Utils.app_name, desc: 'Name of application'

      desc 'config [VAR=value]', 'Show, set or delete environment variables'
      def config(env_var = '')
        Application.config(options[:app], env_var)
      end

      desc 'create', 'Create new application'
      def create
        Application.create(options[:app])
      end

      desc 'destroy', 'Destroy application'
      def destroy
        Application.destroy(options[:app])
      end

      desc 'deployments', 'List all deployments'
      def deployments
        Application.deployments(options[:app])
      end

      desc 'exec COMMAND', 'Execute a one-time-command (e.g. "rails db:migrate")'
      def exec(command)
        Application.exec(options[:app], command)
      end

      desc 'git', 'Show URL of git repository'
      option :quiet, aliases: :'-q', type: :boolean, default: false
      def git
        Application.git(options[:app], options[:quiet])
      end

      desc 'healthcheck [PATH]', 'Show or change the path for the health check'
      def healthcheck(path = '')
        Application.healthcheck(options[:app], path)
      end

      desc 'logs', 'View last log entries of a process'
      option :follow, aliases: :'-f', type: :boolean, default: false,
                      desc: 'Follow live log data'
      option :process, aliases: :'-p', type: :string, default: 'web',
                       desc: 'Process type (valid types are: scheduler, worker and web)'
      def logs
        Application.logs(options[:app], options[:follow], options[:process])
      end

      desc 'maintenance', 'Enable or disable maintenance mode'
      option :on, type: :boolean, default: false, desc: 'Turn on maintenance mode'
      option :off, type: :boolean, default: false, desc: 'Turn off maintenance mode'
      def maintenance
        abort('On and off are mutually exclusive options.') if options[:on] && options[:off]
        if options[:on]
          mode = true
        elsif options[:off]
          mode = false
        end
        Application.maintenance(options[:app], mode)
      end

      desc 'ps', 'Show running processes'
      def ps
        Application.ps(options[:app])
      end

      desc 'rebuild', 'Rebuild app from git repo'
      def rebuild
        Application.rebuild(options[:app])
      end

      desc 'restart', 'Restart application (rolling update)'
      def restart
        Application.restart(options[:app])
      end

      desc 'rollback COMMIT', 'Rollback to a previous deployment specified by its commit hash'
      def rollback(commit)
        Application.rollback(options[:app], commit)
      end

      desc 'scale #REPLICA', 'Scale up or down (horizontal scaling)'
      def scale(replica)
        Application.scale(options[:app], replica)
      end

      desc 'start', 'Start application'
      def start
        Application.start(options[:app])
      end

      desc 'status', 'Status of application'
      def status
        Application.status(options[:app])
      end

      desc 'stop', 'Stop application'
      def stop
        Application.stop(options[:app])
      end
    end
  end
end
