module Gleis
  module CLI
    # Database-related CLI subcommands
    class Db < Thor
      class_option :app, aliases: :'-a', type: :string, default: Utils.app_name, desc: 'Name of application'

      desc 'backup', 'Backup locally database DATABASE_URL'
      def backup
        Gleis::Database.backup(options[:app])
      end

      desc 'delete DB_ENV_VAR', 'Delete database configured at variable name'
      def delete(env_var_name)
        Gleis::Database.delete(options[:app], env_var_name)
      end

      desc 'info', 'Information about database DATABASE_URL'
      def info
        Gleis::Database.info(options[:app])
      end

      desc 'new', 'Create new database'
      def new
        Gleis::Database.new(options[:app])
      end

      desc 'promote DB_ENV_VAR', 'Promote database to app using variable name (e.g. DATABASE1_URL)'
      def promote(env_var_name)
        Gleis::Database.promote(options[:app], env_var_name)
      end

      desc 'psql', 'Connect to PostgreSQL database DATABASE_URL'
      def psql
        Gleis::Database.psql(options[:app])
      end

      desc 'push LOCAL_DB_NAME', 'Copy local database to remote database DATABASE_URL'
      def push(local_name)
        Gleis::Database.push(options[:app], local_name)
      end

      desc 'reset DB_ENV_VAR', 'Reset database configured at variable name'
      def reset(env_var_name)
        Gleis::Database.reset(options[:app], env_var_name)
      end
    end
  end
end
